
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _MOOF_RANDOM_HH_
#define _MOOF_RANDOM_HH_

/**
 * @file Random.hh
 * Simple pseudo-random number generators.
 */


namespace Mf {

/**
 * Provide the RNG with a seed.
 * @param theSeed
 * @return theSeed
 */

unsigned setSeed(unsigned theSeed);

/**
 * Seed the RNG with the current time. This is good enough in most cases.
 * @return The seed used.
 */

unsigned setSeed();


/**
 * The most generic generator gets enough random bits to fill the integer type.
 * @return Random value of maximum domain for type T.
 */

template <typename T>
T getRandom();


/**
 * Get a random boolean value.
 * @return True or false.
 */

template <>
bool getRandom<bool>();


/**
 * Get a random integer value with a limited domain.
 * @param lower Smallest possible value.
 * @param upper Largest possible value.
 * @return Integer between lower and upper, inclusive.
 */

template <typename T>
T getRandom(T lower, T upper);


/**
 * Get a random float with a limited domain.
 * @param lower Smallest possible value.
 * @param upper Largest possible value.
 * @return Float between lower and upper, inclusive.
 */

template <>
float getRandom(float lower, float upper);

/**
 * Get a random double with a limited domain.
 * @param lower Smallest possible value.
 * @param upper Largest possible value.
 * @return Double between lower and upper, inclusive.
 */

template <>
double getRandom(double lower, double upper);

/**
 * Get a random float with a domain limited to [0.0, 1.0].
 * @return Float between 0.0 and 1.0, inclusive.
 */

template <>
float getRandom();

/**
 * Get a random double with a domain limited to [0.0, 1.0].
 * @return Double between 0.0 and 1.0, inclusive.
 */

template <>
double getRandom();


} // namespace Mf


#endif // _MOOF_RANDOM_HH_

/** vim: set ts=4 sw=4 tw=80: *************************************************/

